/***********************************************************************
* File Name    : r_IICA0_LIB.c
* programed by : `R
* Device(s)    : R5F100LG
* Tool-Chain   : CC-RL
* Description  : This file implements device driver for IICA0 slave mode.
* Creation Date: 2015/11/09
* Modified Date: 2016/06/10
************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"

/***********************************************************************
Pragma directive
************************************************************************/
#pragma interrupt r_iica0_interrupt(vect=INTIICA0,bank=RB3)

/***********************************************************************
Global variables , constant and functions
************************************************************************/
#define     DATA_MAX        ( 4 )           /* data number to transfer      */
#define     LED_MAX         ( 2 )           /* data number to display LED   */

#define     TX_LIMIT        ( DATA_MAX -1 ) /* transmit buffer mask data    */
#define     RX_LIMIT        ( LED_MAX - 1 ) /* receive buffer mask data     */
#define     RAM_BASE        ( 0x80 )        /* RAM is located 0x80 -0xFF    */
#define     RAM_MASK        ( RAM_BASE - 1 )/* RAM address mask data        */

#define     COM_FLAG_MASK   ( 0b00001111 )  /* communication flag clear data*/
#define     SLAVE_MODE      ( 0 )           /* start as slave mode          */
#define     DETECT          ( 1 )           /* detect flag                  */
#define     SELECTED        ( 1 )           /* selected by master           */
#define     NOT_SELECTED    ( 0 )           /* not selected by master       */
#define     RECEIVE_MODE    ( 0 )           /* IIC bus receive operation    */
#define     TRANSMIT_MODE   ( 1 )           /* IIC bus transmit operation   */
#define     NACK            ( 0 )           /* not ACK from master          */

uint8_t     static g_ram_area[128];         /* 128byte RAM area for master  */
uint8_t     static g_rx_data[DATA_MAX];     /* received data buffe          */
uint16_t    static g_tx_data[DATA_MAX];     /* transmit data buffer         */
volatile    uint8_t static g_low_data_temp; /* low byte data for transmit   */
volatile    uint8_t static g_low_data_index;/* show next transmit is low    */

volatile    uint8_t static gp_trx_data;     /* transfer data store pointer  */

volatile    uint8_t g_det_stop;             /* stop condition detect flag   */
volatile    uint8_t f_regadr;
struct  sts_b{
    uint8_t spdf:1;                         /* Stop condition detect flag   */
    uint8_t stdf:1;                         /* Start condition detect flag  */
    uint8_t ackdf:1;                        /* ACK detect flag              */
    uint8_t trcf:1;                         /* transmit flag                */
    uint8_t coif:1;                         /* selected by address flag     */
    uint8_t excf:1;                         /* extension code reception flag*/
    uint8_t aldf:1;                         /* arbitration loss flag        */
    uint8_t mstsf:1;                        /* Master status flag           */
};

struct sts_b sb;

union   {
    uint8_t flag;
    struct sts_b sb;
}   g_iics;


void R_IICA0_Init(void);                    /* IICA0 buffer initialize      */
uint8_t R_IICA0_Get(uint8_t ptr);           /* get IICA0 received data      */
void R_IICA0_Put(uint8_t ptr,uint16_t data); /* set data to TX buffer       */
uint8_t R_IICA0_Status(void);               /* check IICA0 status           */

/***********************************************************************
* Function Name: R_IICA0_Init
* Description  : This function is IICA0 data initialize routine.
* Arguments    : None
* Return Value : None
************************************************************************/
void R_IICA0_Init(void)
{
    uint8_t i;

    for( i = 0 ; i < DATA_MAX ; i++ )       /* clear data buffer            */
    {
        g_rx_data[i] = 0x00;
        g_tx_data[i] = 0x00;
    }

    for( i = 0 ; i < RAM_BASE ; i++ )
    {
        g_ram_area[i] = i;                  /* initialize RAM data          */
    }

    SPIE0 = TRUTH;                          /* enable STOP detect interrupt */
    gp_trx_data = 0x00;                     /* initialize buffer pointer    */

    g_det_stop = 0x00;                      /* not detect stop condition    */
 
}/* End of function R_IICA0_Init() */

/***********************************************************************
* Function Name: R_IICA0_Get
* Description  : This function get RX buffer data routine.
* Arguments    : pointer
* Return Value : data
************************************************************************/
uint8_t R_IICA0_Get(uint8_t ptr)
{

    uint8_t rx_data;

    rx_data = g_rx_data[(ptr & RX_LIMIT)];  /* read data from buffer      */
    return( rx_data );

}/* End of function R_IICA0_Get() */

/***********************************************************************
* Function Name: R_IICA0_Status
* Description  : This function get IICA0 status and clear STOP cond flag
* Arguments    : None
* Return Value : status
************************************************************************/
uint8_t R_IICA0_Status(void)
{

    uint8_t status;

    status = g_iics.flag;                   /* read IICA0  status           */

    return( status );

}/* End of function R_IICA0_Status() */


/***********************************************************************
* Function Name: R_IICA0_Put
* Description  : This function set data to TX buffer.
* Arguments    : pointer,data
* Return Value : None
************************************************************************/
void R_IICA0_Put(uint8_t ptr,uint16_t data)
{

    uint8_t wpt;

    wpt = ptr & TX_LIMIT;                   /* mask non used bit            */

    IICAMK0 = INT_MASK;                     /* for exclusive control        */
    g_tx_data[wpt] = data;                  /* store data in Tx buffer      */
    IICAMK0 = INT_ENABLE;                   /* enable INTIICA0              */

}/* End of function R_IICA0_Put() */


/***********************************************************************
* Function Name: r_iica0_interrupt
* Description  : This function is INTIICA0 interrupt service routine.
* Arguments    : None
* Return Value : None
************************************************************************/
static void r_iica0_interrupt(void)
{

    uint16_t datawork;                      /* for transmit data (temp)     */

    g_iics.flag = IICS0;                       /* read IICA0 status            */

    EI();                                   /* multiple interrupt enable*/

    if( SLAVE_MODE == g_iics.sb.mstsf )
    {
        if( DETECT == g_iics.sb.spdf )      /* check STOP condition         */
        {                                   /* STOP condition is detected   */

/*--------------------------------------
    STOP condition detect interrupt
--------------------------------------*/

            g_det_stop = DETECT_STOP;        /* set STOP condition flag      */

        }                                   /* and exit from communication  */
        else                                /* not detect STOP condition    */
        {

/*--------------------------------------
    data transfer end interrupt timing
--------------------------------------*/

            if( DETECT == g_iics.sb.stdf )  /* check START condition        */
            {                               /* detect START condition       */
                if( NOT_SELECTED == g_iics.sb.coif ) /* check selected or not */
                {                           /* not selected and exit        */
                    LREL0 = 1;              /* exit from communication      */
                }
                else
                {

/*--------------------------------------
    selected by slave address
--------------------------------------*/

                    if( RECEIVE_MODE == g_iics.sb.trcf ) /* check transmit or receive */
                    {

/*--------------------------------------
    receive mode start
    (start register address)
--------------------------------------*/
                        f_regadr = TRUTH;   /* wait for register address  */
                        WREL0 = TRUTH;      /* Cancel wait & start receive  */

                    }
                    else
                    {

                        if ( RAM_BASE > gp_trx_data )
                        {
/*--------------------------------------
    data transmit mode start
    (transmit from high byte)
--------------------------------------*/

                            g_low_data_index = TRUTH; /* set low byet ready     */
                            datawork = g_tx_data[gp_trx_data];
                            IICA0 = (datawork >> 8);  /* transmit high byte     */
                            g_low_data_temp = datawork & 0xFF;
                            gp_trx_data++;
                            gp_trx_data &= TX_LIMIT;
                        }
                        else
                        {
/*--------------------------------------
    RAM data transmit mode start
--------------------------------------*/
                            IICA0 = g_ram_area[(gp_trx_data & RAM_MASK)];
                            gp_trx_data++;      /* move pointer to next address */
                            gp_trx_data |= RAM_BASE;
                        }

                    }
                }
            }
            else
            {

/*--------------------------------------
    data transfer timing
--------------------------------------*/

                if( RECEIVE_MODE == g_iics.sb.trcf ) /* check direction     */
                {

/*--------------------------------------
    receive timing
--------------------------------------*/

                    if( TRUTH == f_regadr ) /* check reg addr timing        */
                    {

/*--------------------------------------
    register address receive timing
--------------------------------------*/

                        gp_trx_data = IICA0; /* set register address        */
                        f_regadr = FALSE;   /* clear reg addr flag          */

                    }
                    else
                    {

/*--------------------------------------
    data receive timing
--------------------------------------*/

                        if ( RAM_BASE > gp_trx_data )
                        {
                            
                            g_rx_data[gp_trx_data] = IICA0; /* store received data */
                            gp_trx_data++;      /* move pointer to next addr    */
                            gp_trx_data &= RX_LIMIT;
                        }
                        else
                        {
/*--------------------------------------
    data receive and store it to RAM
--------------------------------------*/
                            g_ram_area[gp_trx_data & RAM_MASK] = IICA0;
                            gp_trx_data++;      /* move pointer to next addr    */
                            gp_trx_data |= RAM_BASE;
                        }

                    }
                    
                    WREL0 = TRUTH;          /* Cancel wait & start receive  */
                }
                else
                {

/*--------------------------------------
    data transmit timing
--------------------------------------*/

                    if( NACK == g_iics.sb.ackdf ) /* check communication end*/
                    {

/*--------------------------------------
    communication end(NACK from master)
--------------------------------------*/

                        LREL0 = TRUTH;      /* exit from communication      */
                    }
                    else
                    {

/*--------------------------------------
    transmit next data to master
--------------------------------------*/
                        if ( RAM_BASE > gp_trx_data )
                        {


/*--------------------------------------
    transmit next A/D data to master
--------------------------------------*/
                            if ( TRUTH == g_low_data_index )
                            {               /* low byte transmit timing     */

                                IICA0 = g_low_data_temp; /* transmit low byte   */
                                g_low_data_index = FALSE;

                            }
                            else
                            {               /* high byte transmit timing    */

                                g_low_data_index = TRUTH; /* set low byet ready */
                                datawork = g_tx_data[gp_trx_data];
                                IICA0 = (datawork >> 8);  /* transmit high byte */
                                g_low_data_temp = datawork & 0xFF;
                                gp_trx_data++;
                                gp_trx_data &= TX_LIMIT;

                            }
                        }
                        else
                        {
/*--------------------------------------
    transmit next RAM data to master
--------------------------------------*/
                            IICA0 = g_ram_area[(gp_trx_data & RAM_MASK)];
                            gp_trx_data++;
                            gp_trx_data |= RAM_BASE;
                        }
                    }
                }
            }
        }
    }
    else
    {
        /* do noting */
    }

}/* End of interrupt function r_iica0_interrupt() */

