/***********************************************************************
* File Name    : r_inttm.c
* programed by : `R
* Device(s)    : R5F10267
* Tool-Chain   : CC-RL
* Description  : This file implements device driver for TAU module.
* Creation Date: 2015/11/09
* Modified Date: 2016/06/10
************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_led_disp.h"

/***********************************************************************
Pragma directive
************************************************************************/
#pragma interrupt r_tau0_channel0_interrupt(vect=INTTM00,bank=RB1)

/***********************************************************************
Global variables and functions
************************************************************************/
#define     DISP_OFF_DATA   (0b00000110)    /* display off data for P4      */
#define     TIMING1         (0b00000100)    /* display timing1 data for P4  */
#define     TIMING2         (0b00000010)    /* display timing2 data for P4  */
#define     KEY_TIMING      (10)            /* key check every 50ms         */
#define     DATA_MASK       (DATA_NUMBER-1) /* mask data for pointer        */

uint8_t static g_disp_data_bf[DATA_NUMBER]; /* display data buffer area     */
uint8_t static g_sel_data;                  /* display data select          */

uint8_t static g_disp_timing;               /* display data timing          */

/*******************************************************************
* Function Name: R_DISP_Init
* Description  : This function initialize display data image
* Arguments    : None
* Return Value : None
********************************************************************/
void R_DISP_Init(void)
{
    uint8_t i;

    for ( i = 0 ; i < DATA_NUMBER ; i++ )
    {
        g_disp_data_bf[i] = 0x00;           /* clear display buffer         */
    }

    g_sel_data = 0x00;                      /* select display data0         */
    g_disp_timing = 0x00;                   /* initialize display timing    */

}/* End of function R_DISP_Init() */

/*******************************************************************
* Function Name: R_Make_DISP_data
* Description  : This function set display data in buffer
* Arguments    : CH_No,in_data
* Return Value : None(g_disp_data)
********************************************************************/
void R_Make_DISP_data(uint8_t CH_No,uint8_t in_data)
{

    g_disp_data_bf[(DATA_MASK & CH_No)] = in_data;

}/* End of function R_Make_DISP_data() */

/***********************************************************************
* Function Name: r_tau0_channel0_interrupt
* Description  : This function is INTTM00 interrupt service routine.
* Arguments    : None
* Return Value : None
************************************************************************/
static void r_tau0_channel0_interrupt(void)
{

    uint8_t disp_work;

    EI();                                   /* multiple interrupt enable*/

    P4 |= DISP_OFF_DATA;                    /* clear display            */

    disp_work = g_disp_data_bf[g_sel_data]; /* get display data         */

    if ( FALSE == ( g_disp_timing & 0x01 ) )
    {                                       /* display timing 1         */
        P1 = ( disp_work >> 4 );            /* set high nibble of data  */
        P4 ^= TIMING1;                      /* drive P42 to low         */
    }
    else
    {                                       /* display timing 2         */
        P1 = ( disp_work & 0x0F );          /* set low nibble of data   */
        P4 ^= TIMING2;                      /* drive P41 to low         */
    }

/*
    display timing cout up
*/
    g_disp_timing++;                        /* count up timing counter  */

    if ( KEY_TIMING <= g_disp_timing )      /* check key check timing   */
    {
        g_disp_timing = 0x00;               /* initialize display timing*/

        if ( FALSE == P13_bit.no7 )
        {                                   /* key switch is pushed     */
            g_sel_data = 0x01;              /* select display data2     */
        }
        else
        {                                   /* key switch is not pushed */
            g_sel_data = 0x00;              /* select display data1 */
        }
    }
    else
    {
        /* do nothing */
    }

}/* End of interrupt function r_tau0_channel0_interrupt() */

/***********************************************************************
* Function Name: R_TAU0_Channel0_Start
* Description  : This function starts TAU0 channel 0 counter.
* Arguments    : None
* Return Value : None
************************************************************************/
void R_TAU0_Channel0_Start(void)
{
    TMIF00 = FALSE;                     /* clear INTTM00 interrupt flag */
    TMMK00 = INT_ENABLE;                /* enable INTTM00 interrupt */
    TS0 |= 0x01;                        /* start TM00 operation     */

}/* End of function R_TAU0_Channel0_Start() */
